#include <iostream>
#include <string>
#include <time.h>
#include <fitsio.h>
#include "PSRFits.hh"
#include "Utils.hh"
#include "Block.hh"
#include "Header.hh"

#define USE_X true

using namespace Block;
using namespace Header;

namespace FITS
{
  
  inline void FITSError(std::string function, int status)
  {
    std::cerr << "\aFITSError <" << function << "> (status="<<status<<"): ";
    fits_report_error(stderr, status);
  }
 //--------------------------------------------------------------------------

  PSRFits::PSRFits()
  {
    this->newFile = true;
    this->filestate = CLOSED;
  }
 //--------------------------------------------------------------------------

  PSRFits::PSRFits(std::string name)
  {
    this->filestate = CLOSED;
    this->openFile(name);
  }
 //--------------------------------------------------------------------------

  PSRFits::PSRFits(DataBlock &block)
  {
    this->filestate = CLOSED;
    this->openFile(block);
  }
 //--------------------------------------------------------------------------

  void PSRFits::setFITSname(Header::SURVEY surveyType)
  {
    std::string date;
    int j=0;
    for(int i=0;i<this->DATE_initial.size();i++)
      if(DATE_initial[i]!='/') date += this->DATE_initial[i];

    std::string time = removeLeadingBlanks(this->UT_initial);
    time = decToDMS( dmsToDec(time), 0, "");
    
    std::stringstream output;
    if(surveyType == S70) output << "S70_";
    else if(surveyType == PM) output << "PM_";
    else if(surveyType == PH) output << "PH_";
    else std::cerr << "Error <PSRFits::setFITSname> : unknown survey!\n";

    output << date << "_" << time;

    if((surveyType == PM)||(surveyType == PH))
      output << "_" 
	     << std::setw(2) << std::setfill('0') 
	     << this->beam_number;

    output << ".fits";

    this->FITSname = output.str();
    
  }
 //--------------------------------------------------------------------------

  int PSRFits::openFile()
  {
    return this->openFile(this->FITSname);
  }
 //--------------------------------------------------------------------------

  int PSRFits::openFile(DataBlock &block)
  {
//     std::string name = 
//       getIAUName(dmsToDec(block.getKeyword<std::string>(RA))*15.,
// 		 dmsToDec(block.getKeyword<std::string>(DEC)));
//     name += ".fits";
    return this->openFile(block.getFITSname());
  }
 //--------------------------------------------------------------------------

  int PSRFits::openFile(std::string name)
  {
    /**
     *  int PSRFits::openFile(string)
     *   Opens the file named by the argument.
     *   If the file exists, it is opened.
     *   If not, it is created using the correct template file.
     *    In the latter case, the fix function is called, and the file is
     *    flagged as "new".
     */
    int status = 0, exists;
    fits_file_exists(name.c_str(),&exists,&status);
    if(status) FITSError("PSRFits::openFile 1",status);
    status=0;
    if(exists<=0){
      char *template_file = (char *)getPSRFITSdefn().c_str();
      fits_create_template(&this->fptr, name.c_str(), template_file, &status);
      std::cerr << "Creating new file " << name << "\n";
    }
    else{
      this->newFile = false;
      fits_open_file(&this->fptr, name.c_str(), READWRITE, &status);
    }
    if(status) FITSError("PSRFits::openFile 2",status);
    else{
      if(exists<=0){
	this->newFile = true;
	this->fixPSRFITS();
      }
      this->gotoPHU();
      this->filestate = OPEN;
    }
    return status;
  }
 //--------------------------------------------------------------------------

  void PSRFits::fixPSRFITS()
  {
    /**
     *  void PSRFits::fixPSRFITS()
     *   Corrects errors that exist in the PSRFITS definition:
     *      - set the TFORM keyword for the data column in the SUBINT table
     *      - fix the units for the RA & DEC columns
     */
    if(this->gotoSubint()){
      std::cerr << 
	"Error <PSRFits::fixPSRFITS> : Could not go to the SUBINT table.\n";
      return;
    }
    
    int status=0;
    char cval[9];
    if(USE_X) strcpy(cval,"16X");
    else      strcpy(cval,"2B");
    fits_update_key(this->fptr, TSTRING, "TFORM19", cval, NULL, &status);
    if(status) FITSError("PSRFits::fixPSRFITS 1", status);
    status=0;
    strcpy(cval,"deg");
    fits_update_key(this->fptr, TSTRING, "TUNIT6", cval, NULL, &status);
    if(status) FITSError("PSRFits::fixPSRFITS 2", status);
    status=0;
    fits_update_key(this->fptr, TSTRING, "TUNIT7", cval, NULL, &status);
    if(status) FITSError("PSRFits::fixPSRFITS 3", status);
    status=0;
    strcpy(cval,"MHz");
    fits_update_key(this->fptr, TSTRING, "TUNIT15", cval, NULL, &status);
    if(status) FITSError("PSRFits::fixPSRFITS 4", status);

    this->gotoPHU();
  }

 //--------------------------------------------------------------------------
  
  int PSRFits::closeFile()
  {
    /**
     *  PSRFits::closeFile()
     *   Closes the file if it is open. Adjusts the filestate variable as well.
     */
    int status = 0;
    if(this->filestate == OPEN){
      fits_close_file(this->fptr, &status);
      if(status) FITSError("PSRFits::closeFile",status);
      this->filestate = CLOSED;
    }
    return status;
  }
 //--------------------------------------------------------------------------
  
  int PSRFits::gotoPHU()
  {
    /**
     *  int PSRFits::gotoPHU()
     *   Moves to the primary header unit
     */
    int status = 0;
    fits_movabs_hdu(this->fptr, 1, NULL, &status);    
    if(status) FITSError("PSRFits::gotoPHU",status);
    return status;
  }
 //--------------------------------------------------------------------------
  
  int PSRFits::gotoSubint()
  {
    /**
     *  int PSRFits::gotoSubint()
     *   Moves to the SUBINT binary table extension.
     */
    int status = 0;
    char name[8];
    strcpy(name,"SUBINT");
//     fits_movnam_hdu(this->fptr, BINARY_TBL, "SUBINT", 0, &status);
    fits_movnam_hdu(this->fptr, BINARY_TBL, name, 0, &status);
    if(status) FITSError("PSRFits::gotoSubint",status);
    return status;
  }
 //--------------------------------------------------------------------------
  
  /**
   *  int PSRFits::updateKeyword(name, value)
   *   Updates the keyword "name" in the current header unit, giving
   *    it the value given by "value".
   *   A templated function that should work for all cases.
   */
  template <typename T> int PSRFits::updateKeyword(std::string name, T value)
  {
    int status=0;
    fits_update_key(this->fptr, TINT, name.c_str(), &value, NULL, &status);
    if(status) FITSError("PSRFits::updateKeyword",status);
    return status;
  }
  template <> int PSRFits::updateKeyword(std::string name, int value){
    int status=0;
    fits_update_key(this->fptr, TINT, (char *)name.c_str(), 
		    &value, NULL, &status);
    if(status) FITSError("PSRFits::updateKeyword",status);
    return status;
  }
  template <> int PSRFits::updateKeyword(std::string name, long value){
    int status=0;
    fits_update_key(this->fptr, TLONG, (char *)name.c_str(), 
		    &value, NULL, &status);
    if(status) FITSError("PSRFits::updateKeyword",status);
    return status;
  }
  template <> int PSRFits::updateKeyword(std::string name, float value){
    int status=0;
    fits_update_key(this->fptr, TFLOAT, (char *)name.c_str(),
		    &value, NULL, &status);
    if(status) FITSError("PSRFits::updateKeyword",status);
    return status;
  }
  template <> int PSRFits::updateKeyword(std::string name, double value){
    int status=0;
    fits_update_key(this->fptr, TDOUBLE, (char *)name.c_str(), 
		    &value, NULL, &status);
    if(status) FITSError("PSRFits::updateKeyword",status);
    return status;
  }
  template <> int PSRFits::updateKeyword(std::string name, std::string value){
    int status=0;
    fits_update_key(this->fptr, TSTRING, (char *)name.c_str(), 
		    (char *)value.c_str(), NULL, &status);
    if(status) FITSError("PSRFits::updateKeyword",status);
    return status;
  }
  template <> int PSRFits::updateKeyword(std::string name, char *value){
    int status=0;
    fits_update_key(this->fptr, TSTRING, (char *)name.c_str(), 
		    value, NULL, &status);
    if(status) FITSError("PSRFits::updateKeyword",status);
    return status;
  }
 //--------------------------------------------------------------------------

  int PSRFits::getColnum(char *name)
  {
    /**
     *  int PSRFits::getColnum()
     *    Returns the number of the column called "name".
     */
    int status=0,colnum=0;
    fits_get_colnum(this->fptr, CASEINSEN, name, &colnum, &status);
    if(status) FITSError("getColnum", status);
    return colnum;
  }
  
  long PSRFits::getNumRows()
  {
    /**
     *  long PSRFits::getNumRows()
     *    Returns the number of rows in the current binary table.
     */
    int status=0;
    long nrows=0;
    fits_get_num_rows(this->fptr, &nrows, &status);
    if(status) FITSError("getNumRows", status);
    return nrows;
  }
 //--------------------------------------------------------------------------
  
  /**
   *  int PSRFits::writeColumnValue(name, value)
   *   Writes a new value in the column "name" in the current binary table.
   *   It adds a single value at the bottom of the table (ie. in the largest
   *    numbered row).
   *   A templated function that should work for all cases.
   */
  template <class T> int PSRFits::writeColumnValue(char *name, T value)
  {
    int status=0;
    int colnum = this->getColnum(name);
    long nrows = this->getNumRows();
    fits_write_col(this->fptr, TINT, colnum, nrows, 1, 1, &value, &status);
    if(status) FITSError("PSRFits::writeColumnValue", status);
    return status;
  }
  template <> int PSRFits::writeColumnValue(char *name, long value)
  {
    int status=0;
    int colnum = this->getColnum(name);
    long nrows = this->getNumRows();
    fits_write_col(this->fptr, TLONG, colnum, nrows, 1, 1, &value, &status);
    if(status) FITSError("PSRFits::writeColumnValue", status);
    return status;
  }
  template <> int PSRFits::writeColumnValue(char *name, float value)
  {
    int status=0;
    int colnum = this->getColnum(name);
    long nrows = this->getNumRows();
    fits_write_col(this->fptr, TFLOAT, colnum, nrows, 1, 1, &value, &status);
    if(status) FITSError("PSRFits::writeColumnValue", status);
    return status;
  }
  template <> int PSRFits::writeColumnValue(char *name, double value)
  {
    int status=0;
    int colnum = this->getColnum(name);
    long nrows = this->getNumRows();
    fits_write_col(this->fptr, TDOUBLE, colnum, nrows, 1, 1, &value, &status);
    if(status) FITSError("PSRFits::writeColumnValue", status);
    return status;
  }
 //--------------------------------------------------------------------------

  int PSRFits::startNewRow()
  {
    /**
     *  int PSRFits::startNewRow()
     *    Adds a new row to the bottom of the current binary table.
     *    Also adds the row number as the ISUBINT column value.
     */
    int nrows = this->getNumRows();
    int status = 0;
    fits_insert_rows(this->fptr, nrows, 1, &status);
    this->writeColumnValue("ISUBINT", nrows+1);
    if(status) FITSError("PSRFits::startNewRow", status);
    return status;
  }
 //--------------------------------------------------------------------------

  void PSRFits::writePHU(DataBlock &block)
  {
    /**
     *  void PSRFits::writePHU(block)
     *   Writes the header information contained in the DataBlock block
     *    to the primary header unit.
     *   At the end the current header unit will be the PHU.
     */
    this->gotoPHU();

    char *todaysdate     = new char[25];
    int timeref,status=0;
    fits_get_system_time(todaysdate, &timeref, &status);
    if(status) FITSError("PSRFits::writePHU", status);
    std::string projid   = block.getSurveyName();
    std::string telescop = block.getKeyword<std::string>(TELID);
    // CHECK -- the following PHU keywords are all survey dependent
    // CHECK -- need to find correct values.
    double ant_x         = block.Survey().getAntX();
    double ant_y         = block.Survey().getAntY();
    double ant_z         = block.Survey().getAntZ();
    std::string frontend = block.Survey().getFrontend();
    std::string fd_poln  = block.Survey().getFd_poln();
    int fd_hand          = block.Survey().getFd_hand();
    float fd_sang        = block.Survey().getFd_sang();
    float fd_xyph        = block.Survey().getFd_xyph();
    std::string backend  = block.Survey().getBackend();
    std::string beconfig = block.Survey().getBeconfig();
    int be_phase         = block.Survey().getBe_phase();
    int be_dcc           = block.Survey().getBe_dcc();
    double tcycle        = block.Survey().getTcycle();
    int nrcvr            = block.getKeyword<int>(NFILTER);
    std::string obs_mode = PHU_kw_obs_mode;
    std::string src_name = deblank(block.getKeyword<std::string>(PSRNAME));
    std::string coord_md = PHU_kw_coord_md;
    //  CHECK -- NEED TO DECIDE ON RA/DEC OR GL/GB
    std::string stt_crd1 = deblank(block.getKeyword<std::string>(RA));
    std::string stt_crd2 = deblank(block.getKeyword<std::string>(DEC));
    std::string trk_mode = block.Survey().getTrk_mode();
    std::string stp_crd1 = deblank(block.getKeyword<std::string>(RA));
    std::string stp_crd2 = deblank(block.getKeyword<std::string>(DEC));
    float scanlen        = block.Survey().getScanlen();
    std::string fd_mode  = deblank(block.getKeyword<std::string>(FDCNTRL));
    float fa_req         = block.getKeyword<float>(FEED_ANGLE);
    std::string cal_mode = block.Survey().getCal_mode();
    float cal_freq       = block.Survey().getCal_freq();
    float cal_dcyc       = block.Survey().getCal_dcyc();
    float cal_phs        = block.Survey().getCal_phs();
    // CHECK -- STT_DATE NEEDS TO BE PUT INTO A STANDARD FORMAT.
    std::string stt_date = deblank(block.getKeyword<std::string>(DATE));
    if(stt_date[0] == '0') stt_date = "20" + stt_date;
    else                   stt_date = "19" + stt_date;
    for(int i=0;i<stt_date.size();i++) if(stt_date[i]=='/') stt_date[i]='-';
    std::string stt_time = deblank(block.getKeyword<std::string>(UT));
    long stt_imjd        = block.getKeyword<int>(MJD);
    long stt_smjd        = long(dmsToDec(stt_time) * 3600.);
    double stt_offs      = 0.; //CHECK -- what is STT_OFFS?
    double stt_lst       = dmsToDec(block.getKeyword<std::string>(LST))*3600.;

//     this->UT_initial = stt_time;

    this->updateKeyword("DATE",     todaysdate);
    this->updateKeyword("PROJID",   projid);
    this->updateKeyword("TELESCOP", telescop);
    this->updateKeyword("ANT_X",    ant_x);          
    this->updateKeyword("ANT_Y",    ant_y);        
    this->updateKeyword("ANT_Z",    ant_z);        
    this->updateKeyword("FRONTEND", frontend);
    this->updateKeyword("FD_POLN",  fd_poln);
    this->updateKeyword("FD_HAND",  fd_hand);
    this->updateKeyword("FD_SANG",  fd_sang);
    this->updateKeyword("FD_XYPH",  fd_xyph);
    this->updateKeyword("BACKEND",  backend);
    this->updateKeyword("BECONFIG", beconfig);
    this->updateKeyword("BE_PHASE", be_phase);
    this->updateKeyword("BE_DCC",   be_dcc);
    this->updateKeyword("TCYCLE",   tcycle);
    this->updateKeyword("NRCVR",    nrcvr);
    this->updateKeyword("OBS_MODE", obs_mode);
    this->updateKeyword("SRC_NAME", src_name);
    this->updateKeyword("COORD_MD", coord_md);
    this->updateKeyword("STT_CRD1", stt_crd1);
    this->updateKeyword("STT_CRD2", stt_crd2);
    this->updateKeyword("TRK_MODE", trk_mode);
    this->updateKeyword("STP_CRD1", stp_crd1);
    this->updateKeyword("STP_CRD2", stp_crd2);
    this->updateKeyword("SCANLEN",  scanlen);
    this->updateKeyword("FD_MODE",  fd_mode);
    this->updateKeyword("FA_REQ",   fa_req);
    this->updateKeyword("CAL_MODE", cal_mode);
    this->updateKeyword("CAL_FREQ", cal_freq);
    this->updateKeyword("CAL_DCYC", cal_dcyc);
    this->updateKeyword("CAL_PHS ", cal_phs);
    this->updateKeyword("STT_DATE", stt_date);
    this->updateKeyword("STT_TIME", stt_time);
    this->updateKeyword("STT_IMJD", stt_imjd);
    this->updateKeyword("STT_SMJD", stt_smjd);
    this->updateKeyword("STT_OFFS", stt_offs);
    this->updateKeyword("STT_LST ", stt_lst);
  }
 //--------------------------------------------------------------------------

  int PSRFits::writeSubint(DataBlock &block)
  {
    /**
     *  void PSRFits::writeSubint(block)
     *   Writes the header information and data contained in the DataBlock 
     *    block to the SUBINT binary table extension.
     *   This does the following:
     *     - Update the table headers
     *     - Start a new row.
     *     - Update the single-valued columns for the current row.
     *     - Call writeSupportArrays to update the array-valued columns.
     *     - Call writeDataArray to write the data column.
     *   At the end the current header unit will be the SUBINT extension.
     */
    if(this->gotoSubint()){
      std::cerr << 
	"Error <PSRFits::writeSubint> : Could not go to the SUBINT table.\n";
      return 1;
    }
    
    this->updateKeyword("NCH_FILE", block.getKeyword<int>(NCHAN));
    this->updateKeyword("INT_TYPE", SUBINT_kw_int_type);
    this->updateKeyword("INT_UNIT", SUBINT_kw_int_unit);
    this->updateKeyword("NCH_STRT", SUBINT_kw_nch_strt);

    this->startNewRow(); 
    // CHECK -- need some check of blk_cntr to make sure the number is 
    // CHECK -- incrementing correctly?
    float tsubint = block.getKeyword<float>(BLK_SEC);
    float off_sub;
    // CHECK -- OFF_SUB is the offset, from the start (taken to mean the 
    // CHECK -- starting time listed in the PHU), of the centre of the 
    // CHECK -- current subintegration.
    off_sub = dmsToDec(block.getKeyword<std::string>(UT)) - 
      dmsToDec(this->UT_initial);
    if(off_sub<0.) off_sub += 24.;
    off_sub = off_sub*3600. + block.getKeyword<float>(BLK_SEC)/2.;
    double lst_sub  = dmsToDec(block.getKeyword<std::string>(LST));
    double ra_sub   = dmsToDec(block.getKeyword<std::string>(RA))*15.;
    double dec_sub  = dmsToDec(block.getKeyword<std::string>(DEC));
    double glon_sub = block.getKeyword<double>(GL);
    double glat_sub = block.getKeyword<double>(GB);
    float  fd_ang   = block.getKeyword<float>(FEED_ANGLE);
    float  pos_ang  = 0.;  // CHECK -- NO OTHER WAY TO DETERMINE POS_ANG?
    float  par_ang  = block.getKeyword<float>(PANGLE);
    float  tel_az   = block.getKeyword<float>(TEL_AZ);
    float  tel_zen  = block.getKeyword<float>(TEL_ZEN);
    
    this->writeColumnValue("TSUBINT",  tsubint);
    this->writeColumnValue("OFFS_SUB", off_sub);
    this->writeColumnValue("LST_SUB",  lst_sub);
    this->writeColumnValue("RA_SUB",   ra_sub);
    this->writeColumnValue("DEC_SUB",  dec_sub);
    this->writeColumnValue("GLON_SUB", glon_sub);
    this->writeColumnValue("GLAT_SUB", glat_sub);
    this->writeColumnValue("FD_ANG",   fd_ang);
    this->writeColumnValue("POS_ANG",  pos_ang);
    this->writeColumnValue("PAR_ANG",  par_ang);
    this->writeColumnValue("TEL_AZ",   tel_az);
    this->writeColumnValue("TEL_ZEN",  tel_zen);
    
    this->writeSupportArrays(block); //DAT_FREQ,DAT_WTS,DAT_OFFS,DAT_SCL

    this->writeDataArray(block);
   
    this->gotoPHU();

    return 0;

  }
 //--------------------------------------------------------------------------

  void PSRFits::writeSupportArrays(DataBlock &block)
  {
    /**
     *  void PSRFits::writeSupportArrays(block)
     *   A function to write the multi-valued columns to the SUBINT table.
     *   These columns are:
     *      - DAT_FREQ: the frequencies of each channel
     *      - DAT_WTS:  all set to 1. 
     *      - DAT_OFFS: all set to 0.
     *      - DAT_SCL:  all set to 1.
     *   The current header at the end is still SUBINT.
     */
    int numChannels = block.getKeyword<int>(NCHAN);
    float *dat_freq = new float[numChannels];
    dat_freq[0] = block.getKeyword<float>(FREQ_CHAN1);
    float deltaFreq = block.getKeyword<float>(CHANBW) / float(numChannels);
    for(int i=1;i<numChannels;i++) dat_freq[i] = dat_freq[i-1] + deltaFreq;

    float *dat_wts = new float[numChannels];
    for(int i=0;i<numChannels;i++) dat_wts[i] = 1.;
    float *dat_offs = new float[numChannels];
    for(int i=0;i<numChannels;i++) dat_offs[i] = 0.;
    float *dat_scl = new float[numChannels];
    for(int i=0;i<numChannels;i++) dat_scl[i] = 1.;

    int status = 0;
    long nrows = this->getNumRows();

    int colnum = this->getColnum("DAT_FREQ");
    status=0;
    fits_modify_vector_len (this->fptr, colnum, numChannels, &status);
    if(status) FITSError("PSRFits::writeSupportArray", status);
    status=0;
    fits_write_col(this->fptr, TFLOAT, colnum, nrows, 1, 
		   numChannels, dat_freq, &status);
    if(status) FITSError("PSRFits::writeSupportArrays", status);

    colnum = this->getColnum("DAT_WTS");
    status=0;
    fits_modify_vector_len (this->fptr, colnum, numChannels, &status);
    if(status) FITSError("PSRFits::writeSupportArray", status);
    status=0;
    fits_write_col(this->fptr, TFLOAT, colnum, nrows, 1, 
		   numChannels, dat_wts, &status);
    if(status) FITSError("PSRFits::writeSupportArrays", status);

    colnum = this->getColnum("DAT_OFFS");
    status=0;
    fits_modify_vector_len (this->fptr, colnum, numChannels, &status);
    if(status) FITSError("PSRFits::writeSupportArray", status);
    status=0;
    fits_write_col(this->fptr, TFLOAT, colnum, nrows, 1, 
		   numChannels, dat_offs, &status);
    if(status) FITSError("PSRFits::writeSupportArrays", status);

    colnum = this->getColnum("DAT_SCL");
    status=0;
    fits_modify_vector_len (this->fptr, colnum, numChannels, &status);
    if(status) FITSError("PSRFits::writeSupportArray", status);
    status=0;
    fits_write_col(this->fptr, TFLOAT, colnum, nrows, 1, 
		   numChannels, dat_scl, &status);
    if(status) FITSError("PSRFits::writeSupportArrays", status);

  }
 //--------------------------------------------------------------------------

  void PSRFits::writeDataArray(DataBlock &block)
  {
    /**
     *  void PSRFits::writeDataArray(block)
     *   A function to write the multi-dimensional data column to the SUBINT 
     *    binary table.
     *   
     *   The current header at the end is still SUBINT.
     */
    long nrows = this->getNumRows();
    int colnum = this->getColnum("DATA");
    int status = 0;

    int numChan = block.getKeyword<int>(NCHAN,0);
    int numPol  = block.getKeyword<int>(NFILTER);
    // CHECK -- HOW DOES NFILTER COMPARE TO NPOL???
    int numBin  = block.getKeyword<int>(SAMP_GRP,0) * 
      block.getKeyword<int>(GRP_BLK);

    int dataPerByte = 8 / block.getKeyword<int>(NBITS,0);


    int numData = numChan * numPol * numBin;
    //CHECK -- IS THIS CONVERSION TO BYTES CORRECT?
    int numBytes = numData / dataPerByte;   

    unsigned char *data = (unsigned char *)block.getData();

    //Fixing size of vector
    if(this->newFile){
      status=0;
      if(USE_X) fits_modify_vector_len (this->fptr, colnum, numData, &status);
      else      fits_modify_vector_len (this->fptr, colnum, numBytes, &status);
      if(status) FITSError("PSRFits::writeDataArray 1", status);
    }
 
     long naxis=3,naxes[3];
    // CHECK -- ORDER OF AXES: first axis is quickest-varying one, so this is 
    // CHECK --  the channel axis. Next quickest is the pol axis (equivalent 
    // CHECK --  to the filter? CHECK THIS). Slowest varying is the sample  
    // CHECK --  axis (which has length numBin).
    // CHECK -- IS THE CONVERSION TO BYTES CORRECT? We are dividing the channel
    // CHECK --  axis, as this should be divisible by 8.
    if(USE_X) naxes[0] = numChan; 
    else      naxes[0] = numChan / dataPerByte; 
    naxes[1] = numPol;
    naxes[2] = numBin;
    if(this->newFile){
      status=0;
      // delete the existing "TDIM19 = (*,*,*)"
      fits_delete_key(this->fptr, "TDIM19", &status);
      if(status) FITSError("PSRFits::writeDataArray 2", status);
      status=0;
      fits_write_tdim(this->fptr, colnum, naxis, naxes, &status);
      if(status) FITSError("PSRFits::writeDataArray 3", status);
    }

    status=0;
    fits_write_col_byt(this->fptr, colnum, nrows, 1, numBytes, data, &status);
//fits_write_col(this->fptr, TBYTE, colnum, nrows, 1, numBytes, data, &status);
    if(status) FITSError("PSRFits::writeDataArray 4", status);

    if(USE_X && this->newFile) {
      this->newFile = false;
      status=0;
      fits_delete_rows(this->fptr, nrows+1, 1, &status);
      if(status) FITSError("PSRFits::writeDataArray 5", status);
    }

  

  }


}
