#include "Utils.hh"
#include <time.h>
#include <stdlib.h>
#include <math.h>
#include <string>
#include <iostream>
#include <sstream>
#include <iomanip>

double dmsToDec(std::string input)
{
  /** 
   *  double dmsToDec(string)
   *   Converts a string in the format +12:23:34.45 to a decimal angle in 
   *    degrees.
   *   Assumes the angle given is in degrees, so if passing RA as the argument,
   *   need to multiply by 15 to get the result in degrees rather than hours.
   *   The sign of the angle is preserved, if present.
   */

  std::string dms = removeLeadingBlanks(input);

  bool isNeg = false;
  if(dms[0]=='-') isNeg = true;

  for(int i=0;i<dms.size();i++) if(dms[i]==':') dms[i]=' ';

  std::stringstream ss;
  ss.str(dms);

  std::string deg,min,sec;
  getline(ss,deg,' ');
  getline(ss,min,' ');
  getline(ss,sec);
  char *end;
  double d = strtod(deg.c_str(),&end);
  double m = strtod(min.c_str(),&end);
  double s = strtod(sec.c_str(),&end);  

  double dec = fabs(d) + m/60. + s/3600.;
  if(isNeg) dec = dec * -1.;

  return dec;

}


std::string decToDMS(double input, int secondPrecision, std::string separator)
{
  /**
   * std::string decToDMS(double input, int secondPrecision, 
   *                       std::string separator)
   *   Converts a decimal into a dd:mm:ss.ss format.
   *   This is the general form, where one can specify the degree of precision
   *   of the seconds, and the separating character.
   */ 
  double hour = trunc(input);
  double fraction = fabs(input - hour);
  double min = trunc( fraction * 60. );
  fraction = fraction * 60. - min;
  double sec = fraction * 60.;
  
  int secondWidth = 2;
  if(secondPrecision>0) secondWidth += 1 + secondPrecision;

  std::stringstream output;
  output << std::setw(2) << std::setfill('0') << std::setprecision(0)
	 << hour << separator 
	 << std::setw(2) << std::setfill('0') << std::setprecision(0)
	 << min  << separator ;
  output.setf(std::ios::fixed);
  output << std::setw(secondWidth) << std::setfill('0') 
	 << std::setprecision(secondPrecision) << sec;

  return output.str();
}

std::string decToDMS(double input)
{
  return decToDMS(input,2,":");
}

std::string getIAUName(double ra, double dec)
{
  /**
   * string getIAUName(double, double)
   *  both ra and dec are assumed to be in degrees.
   *  returns name of the form J1234m4321 or J1234p0123
   *  (note -- assumes J2000.0)
   */

  double raHrs = ra / 15.;
  int h = int(raHrs);
  int m = (int)(fmod(raHrs,1.)*60.);
  int s = (int)(fmod(raHrs,1./60.)*3600.);
  std::stringstream ss(std::stringstream::out);
  ss.setf(std::ios::showpoint);
  ss.setf(std::ios::fixed);
  ss << "J"; 
  ss<<std::setw(2)<<std::setfill('0')<<h;
  ss<<std::setw(2)<<std::setfill('0')<<m;
  ss<<std::setw(2)<<std::setfill('0')<<s;
  int sign = int( dec / fabs(dec) );
  double d = dec / sign;
  h = int(d);
  m = (int)(fmod(d,1.)*60.);
  s = (int)(fmod(d,1./60.)*3600.);
  if(sign==1) ss<<"p"; else ss<<"m";
  ss<<std::setw(2)<<std::setfill('0')<<h;
  ss.unsetf(std::ios::showpos);
  ss<<std::setw(2)<<std::setfill('0')<<m;
  ss<<std::setw(2)<<std::setfill('0')<<s;
  return ss.str();
}


time_t my_timegm (struct tm *tm) {
  time_t ret;
  char *tz;

  tz = getenv("TZ");
  setenv("TZ", "", 1);
  tzset();
  ret = mktime(tm);
  if (tz) setenv("TZ", tz, 1);
  else    unsetenv("TZ");
  tzset();
  return ret;
}

std::string addDay(std::string date)
{
  /**
   *  string addDay(string date)
   *   A function to add a day onto the date.
   *   This assumes the format is yy/mm/dd or yymmdd or yy-mm-dd
   *    -- the crucial thing is that the day number is in the last spot.
   *   The format of the date is preserved, as long as it is one of those 3.
   */ 

  struct tm time;
  time.tm_hour=0;
  time.tm_min =0;
  time.tm_sec =1;
  char buf[255];
  std::string output;
  time_t temp;
  if(date.size()==6) {
    strptime(date.c_str(), "%y%m%d", &time);
    time.tm_mday += 1;
    temp = my_timegm(&time);
    strftime(buf, sizeof(buf), "%y%m%d", gmtime(&temp));
  }
  else if(date[2]=='/'){
    strptime(date.c_str(), "%y/%m/%d", &time);
    time.tm_mday += 1;
    temp = my_timegm(&time);
    strftime(buf, sizeof(buf), "%y/%m/%d", gmtime(&temp));
  }
  else if(date[2]=='-'){
    strptime(date.c_str(), "%y-%m-%d", &time);
    time.tm_mday += 1;
    temp = my_timegm(&time);
    strftime(buf, sizeof(buf), "%y-%m-%d", gmtime(&temp));
  }
  else{
    std::cerr << "<addDay> : date (" << date << ") has unknown format!\n";
    return date;
  }
  output = buf;
  return output;
  
}
